# TODO: Add comment
# 
# Author: rhenry2
###############################################################################
library(data.table)

createShockFile = function(scenario){
	
#setting all parameters to default values. If parameters are changed according to ssp then pull value from samp table, if not
#set as default same as in config file or to a value that makes no change  
	
	paramValues = data.table(Year = seq(2010,2100,1),
			trendYearValue = c(rep(0,16),seq(0.2,1,0.2),rep(1,70)),
			TRADE_BARRIER_MULTIPLIER =scenario[,TRADE_BARRIER_MULTIPLIER], 
			TRANSPORT_COST = scenario[,TRANSPORT_COST],
			TECHNOLOGY_CHANGE_ANNUAL_RATE = scenario[,TECHNOLOGY_CHANGE_ANNUAL_RATE],
			MEAT_EFFICIENCY = scenario[,MEAT_EFFICIENCY],
			FERTILISER_COST_PER_T=scenario[,FERTILISER_COST_PER_T],
			OTHER_INTENSITY_COST = scenario[,OTHER_INTENSITY_COST],
			TRANSPORT_LOSSES = 0.05,
			IRRIG_COST_MULTIPLIER = 1.0, 
			RUMINANT_CHANGE_ANNUAL_RATE = 0.0, 
			MONOGASTRIC_CHANGE_ANNUAL_RATE = 0.0,
			IRRIG_EFF_MULTIPLIER = 1.0,
			FINANCIAL_SPECULATION_MULTIPLIER = 1.0)
	
	paramValues[, names(paramValues) := lapply(.SD, as.numeric)]
	
	shocksHappen = data.table(SSP_Scenario = scenario[,SSP_SCENARIO], Scenario = scenario[,Scenario], Year = seq(2010,2100,1), 
			climate =as.logical(rbinom(91,size=1,prob = shockProbs[ssp == substr(scenario[,SSP_SCENARIO],1,4),climate])), 
			cyber=as.logical(rbinom(91,size=1,prob = shockProbs[ssp == substr(scenario[,SSP_SCENARIO],1,4),cyber])), 
			financial =as.logical(rbinom(91,size=1,prob = shockProbs[ssp == substr(scenario[,SSP_SCENARIO],1,4),financial])))
	
	shockValues = data.table(Year = rep(seq(2010,2100,1),6),shock = rep(c("protectionism","automation","diet","cyber","climate","financial"), each=91), 
			TRADE_BARRIER_MULTIPLIER = 0, 
			TRANSPORT_COST = 0,
			TECHNOLOGY_CHANGE_ANNUAL_RATE = 0,
			MEAT_EFFICIENCY = 0,
			FERTILISER_COST_PER_T=0,
			OTHER_INTENSITY_COST = 0,
			TRANSPORT_LOSSES = 0,
			IRRIG_COST_MULTIPLIER = 0, 
			RUMINANT_CHANGE_ANNUAL_RATE = 0, 
			MONOGASTRIC_CHANGE_ANNUAL_RATE = 0,
			IRRIG_EFF_MULTIPLIER = 0,
			FINANCIAL_SPECULATION_MULTIPLIER = 0)
	
	for (trendType in c("protectionism", "automation", "diet")){
		if(as.logical(rbinom(1,size=1,prob =shockProbs[ssp == substr(scenario[,SSP_SCENARIO],1,4),get(trendType)]))){
			for(paramName in shockDistFile[shock == trendType]$pname){
				
				randomValue = drawValue(trendType, paramName)
				#put in records table
				shockValues[shock == trendType, c(paramName) := randomValue * paramValues[,trendYearValue]]
				
				if(shockDistFile[shock == trendType & pname == paramName]$type == 'additive'){
					paramValues[,c(paramName) := get(paramName) + randomValue * trendYearValue]		
				}
				else{
					paramValues[,c(paramName) := get(paramName) * (1 + randomValue*trendYearValue)]
				}
			}		
		}	
	}
	
	
	for(year in paramValues[,Year]){
		for (shockType in names(shocksHappen)[4:6]){
			if(shocksHappen[Year == year, get(shockType)]){	
				for(paramName in shockDistFile[shock == shockType]$pname){
					
					randomValue = drawValue(shockType, paramName)
					#put in records table
					shockValues[shock == shockType & Year == year, c(paramName) := randomValue]
					
					#shock is changed from baseline value not change from previous year 
					if(shockDistFile[shock == shockType & pname == paramName]$type == 'additive'){
						paramValues[Year == year,c(paramName) := get(paramName) + randomValue]
					}
					else{
						paramValues[Year == year,c(paramName) := get(paramName) * (1 + randomValue)]
					}
				}		
			}
		}
	}
	
	#if negative set to zero, or need to change shock distributions so zero can't happen 
	for(col in names(paramValues)) set(paramValues, i=which(paramValues[[col]]<0), j=col, value=0)
	
	ensDir=file.path(scenario[,Ensemble],scenario[,Scenario])
	
	if (dir.exists(file.path(baseOutputDir,ensDir))) {
		print(paste(ensDir, "exists.  Creating shock file"))
		write.csv(paramValues, file.path(baseOutputDir,ensDir,"shocks.csv"), row.names=FALSE, quote=FALSE)
		write.csv(shockValues, file.path(baseOutputDir,ensDir,paste0("shocksRecord_",scenario[,Scenario],".csv")), row.names=FALSE, quote=FALSE)
	}
	else{
		print(paste(ensDir, "does not exist.  Stopping"))
	}
}

drawValue <- function(shockType, paramName){
	
	randomValue = runif(1, abs(shockDistFile[shock == shockType & pname == paramName]$min), 
			abs(shockDistFile[shock == shockType & pname == paramName]$max))
	
	if (shockDistFile[shock == shockType & pname == paramName]$min <0){
		randomValue = randomValue*-1
	}
	
	randomValue
}

plumDirectory = "/exports/csce/eddie/geos/groups/LURG/models/PLUM"
baseDataDir = "/exports/csce/eddie/geos/groups/LURG/models/PLUM/plumv2/data"
baseOutputDir="/exports/csce/eddie/geos/groups/LURG/models/PLUM/output" # "~/Downloads"

shockDistFile = fread(file.path(baseDataDir, "shockDistributions.csv"))
shockProbs = fread(file.path(baseDataDir,"shockProbabilities.csv"))

ensembleFile = commandArgs(trailingOnly = TRUE)[1]

scenarioTable = fread(file.path(plumDirectory,ensembleFile))

for(i in 1:nrow(scenarioTable)){
#first if column exists
	if("SHOCKS_POSSIBLE" %in% colnames(scenarioTable)){
		if(scenarioTable[i,SHOCKS_POSSIBLE]){
			print(paste("Shocks possible in ",scenarioTable[i,Ensemble],scenarioTable[i,Scenario]))
			createShockFile(scenario = scenarioTable[i])
		}
	}
}


